module MdsFileUtils

  class ZipAnalyzer

    MAX_MDS_FILE_SIZE = 5242880 # 5MB

    EXTENSION_REGEX = /\.zip\z/i
    VALID_COMPOSITE_CONTENTS_REGEX = /(\.zip|\/|\\)\z/i

    def initialize(path, original_filename=path)
      @path = path
      @original_filename = original_filename
    end

    def valid_zip?
      return false if @path.blank?
      return false unless File.exists?(@path)
      return false unless has_valid_extension?
      UnzipmeValidator.new(@path).valid_zip?
    end

    def has_valid_extension?
      return false if @original_filename.blank?
      @original_filename =~ EXTENSION_REGEX
    end

    def is_composite?
      return false unless valid_zip?
      file_list.count { | file | file =~ EXTENSION_REGEX } > 0
    end

    def file_list
      @files ||= UnzipmeUnzipper.new(@path).file_list
    end

    def non_mds_data_in_composite?
      file_list.count { | entry | !( entry =~ VALID_COMPOSITE_CONTENTS_REGEX) } > 0
    end

    def too_big_for_mds?
      file_size > MAX_MDS_FILE_SIZE && !is_composite?
    end

    def file_size
      File.size(@path)
    end

  end

end
